const fs = require('fs');
const remote = require('@electron/remote');
const shell = require('electron').shell;
const webFrame = require('electron').webFrame;
const contextBridge = require('electron').contextBridge;
const ipcRenderer = require('electron').ipcRenderer;

let shareData;


contextBridge.exposeInMainWorld(
									'electron',	{
													'createBrowser': (e, args) => createBrowser(e, args),
													'resizeWindow': (e, args) => resizeWindow(e, args),
													'setDisplay': (e, args) => setDisplay(e, args),
													'dataStore': (e, args) => dataStore(e, args),
													'logData': (e, args) => logData(e, args),
													'processData': (e, args) => processData(e, args),
													'openURL': (e, args) => openURL(e, args),
													'getShareData': () => getShareData(),
													'getDisplayInfo': () => getDisplayInfo(),
													'reload': () => reloadMainWindow(),
													'clearCache': () => clearCache(),
													'quit': () => quitApp()
													
												},
									'api', {
												'send': (channel, data) => {
													// Whitelist channels
													let validChannels = [ 'toMain', 'mainLogData', 'mainCreateBrowser' ];

													if (validChannels.includes(channel)) {
														ipcRenderer.send(channel, data);
													}
												},
										   		'receive': (channel, func) => {
											   		let validChannels = ['fromMain'];
											   		if (validChannels.includes(channel)) {
												   		// Strip event as it includes `sender` 
												   		ipcRenderer.on(channel, (event, ...args) => func(...args));
												   	}
												}
											}
							   );


ipcRenderer.on('resized', (e, ...args) => {

	try {

		windowDimensions.textContent = JSON.stringify(args[0]);
	}
	catch(e) {

	}
});


ipcRenderer.on('fromMain', (e, args) => {

	shareData = args;
});


window.addEventListener('DOMContentLoaded', () => {

	const replaceText = (selector, text) => {

		const element = document.getElementById(selector);

		if (element) { element.innerText = text; }
	}

	for (const dependency of ['chrome', 'node', 'electron']) {

		replaceText(`${dependency}-version`, process.versions[dependency])
	}

});


async function getMetrics(data) {

	let res = await ipcRenderer.invoke('get-metrics', data);
}


function createBrowser(hash, data) {

	ipcRenderer.send('mainCreateBrowser', hash, data);
}


function reloadMainWindow() {

	let win = remote.getCurrentWindow();
	win.reload();
}


async function processData(hash) {

	return JSON.parse(JSON.stringify(hash));
}


function logData(hash, data) {

	ipcRenderer.send('mainLogData', hash, data);
}


function clearCache() {

	//alert(JSON.stringify(webFrame.getResourceUsage()));

	let win = remote.getCurrentWindow();

	win.webContents.session.clearCache();
	webFrame.clearCache();
}


function openURL(url) {

	shell.openExternal(url);
}


function quitApp() {

	ipcRenderer.send('app-quit');
}


function getShareData() {

	return shareData;
}


function dataStore(data) {

	ipcRenderer.send('data-store', data);
}


function resizeWindow(width, height) {

	let win = remote.getCurrentWindow();

	win.setSize(width, height);
}


function getDisplayInfo() {

	let win = remote.getCurrentWindow();

	let displays = remote.screen.getAllDisplays();

	let winBounds = win.getBounds();
	let position = win.getPosition();

	let currentScreen = remote.screen.getDisplayNearestPoint({x: winBounds.x, y: winBounds.y});

	let obj = { 'displays': displays, 'currentDisplay': currentScreen, 'position': position };

	return obj;
}


function setDisplay(obj) {

	if (typeof obj !== 'object') {

		return;
	}

	let bounds = obj['bounds'];
	let position = obj['position'];

	let x;
	let y;
	let xPos;
	let yPos;

	let win = remote.getCurrentWindow();

	if (bounds != undefined && bounds != null && bounds != '' && typeof bounds === 'object') {

		x = bounds['x'];
		y = bounds['y'];

		win.setBounds({ 'x': x, 'y': y });
	}

	if (position != undefined && position != null && position != '' && typeof position === 'object') {

		xPos = position['x'];
		yPos = position['y'];

		win.setPosition(xPos, yPos);
	}
}



// Get shared data
ipcRenderer.send('toMain', '');
